@@ -65,31 +65,12 @@ namespace clx {
 			// for client socket
 			explicit basic_socket(const char_type* host, int port) :
 				rawsocket(), addr_(host, port), connect_(false) {
-				try {
-					this->xconnect();
-				}
-				catch (socket_error& e) {
-					this->assign(rawsocket());
-					throw e;
-				}
+				this->xconnect();
 			}
 			
 			explicit basic_socket(const string_type& host, int port) :
 				rawsocket(), addr_(host, port), connect_(false) {
-				try {
-					this->xconnect();
-				}
-				catch (socket_error& e) {
-					this->assign(rawsocket());
-					throw e;
-				}
-			}
-			
-			basic_socket& operator=(const basic_socket& cp) {
-				addr_ = cp.addr_;
-				connect_ = cp.connect_;
-				this->assign(static_cast<rawsocket>(cp));
-				return *this;
+				this->xconnect();
 			}
 			
 			// for server socket
@@ -98,6 +79,13 @@ namespace clx {
 			
 			virtual ~basic_socket() throw() {}
 			
+			basic_socket& operator=(const basic_socket& cp) {
+				addr_ = cp.addr_;
+				connect_ = cp.connect_;
+				this->assign(static_cast<rawsocket>(cp));
+				return *this;
+			}
+			
 			// operator
 			basic_socket& connect(const char_type* host, int port) {
 				addr_.assign(host, port);
@@ -169,13 +157,7 @@ namespace clx {
 			
 			explicit basic_acceptor(int port, int n = nmax) :
 				rawsocket(), addr_(port), nlisten_(nmax) {
-				try {
-					this->xopen();
-				}
-				catch (socket_error& e) {
-					this->assign(rawsocket());
-					throw e;
-				}
+				this->xopen();
 			}
 			
 			virtual ~basic_acceptor() throw() {}
