@@ -2,7 +2,7 @@
 /*
  *  unix/once.h
  *
- *  Copyright (c) 2004 - 2008, clown. All rights reserved.
+ *  Copyright (c) 2004 - 2009, clown. All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions
@@ -33,7 +33,7 @@
  *  accompanying file LICENSE_1_0.txt or copy at
  *  http://www.boost.org/LICENSE_1_0.txt)
  *
- *  Last-modified: Tue 12 Aug 2008 16:17:00 JST
+ *  Last-modified: Wed 27 May 2009 14:35:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_UNIX_ONCE_H
@@ -49,27 +49,49 @@ namespace clx {
 	typedef unsigned long once_flag;
 	
 	namespace detail {
-		once_flag once_global_epoch = ULONG_MAX;
-		pthread_mutex_t once_epoch_mutex = PTHREAD_MUTEX_INITIALIZER;
-		pthread_cond_t once_epoch_cv = PTHREAD_COND_INITIALIZER;
+		struct global_condition {
+			once_flag flag;
+			pthread_mutex_t mutex;
+			pthread_cond_t cond;
+			
+			global_condition() :
+				flag(ULONG_MAX),
+				mutex(PTHREAD_MUTEX_INITIALIZER),
+				cond(PTHREAD_COND_INITIALIZER) {}
+		};
 		
-		pthread_key_t epoch_tss_key;
-		pthread_once_t epoch_tss_key_flag = PTHREAD_ONCE_INIT;
+		inline global_condition& get_global_condition() {
+			static global_condition cv;
+			return cv;
+		}
+		
+		inline pthread_once_t& get_epoch_tss_once_flag() {
+			static pthread_once_t flag = PTHREAD_ONCE_INIT;
+			return flag;
+		}
+		
+		inline pthread_key_t& get_epoch_tss_key() {
+			static pthread_key_t key;
+			return key;
+		}
 		
 		inline void delete_epoch_tss_data(void* data) {
 			free(data);
 		}
 		
 		inline void create_epoch_tss_key() {
-			pthread_key_create(&epoch_tss_key, delete_epoch_tss_data);
+			pthread_key_t& key = get_epoch_tss_key();
+			pthread_key_create(&key, delete_epoch_tss_data);
 		}
 		
 		inline once_flag& get_once_per_thread_epoch() {
-			pthread_once(&epoch_tss_key_flag, create_epoch_tss_key);
-			void* data = pthread_getspecific(epoch_tss_key);
+			pthread_once_t& flag = get_epoch_tss_once_flag();
+			pthread_key_t& key = get_epoch_tss_key();
+			pthread_once(&flag, create_epoch_tss_key);
+			void* data = pthread_getspecific(key);
 			if (!data) {
 				data = std::malloc(sizeof(once_flag));
-				pthread_setspecific(epoch_tss_key, data);
+				pthread_setspecific(key, data);
 				*static_cast<once_flag*>(data) = ULONG_MAX;
 			}
 			return *static_cast<once_flag*>(data);
@@ -91,28 +113,29 @@ namespace clx {
 		once_flag const epoch = flag;
 		once_flag& this_thread_epoch = detail::get_once_per_thread_epoch();
 		
+		detail::global_condition& inf = detail::get_global_condition();
 		if (epoch < this_thread_epoch) {
-			pthread_mutex_lock(&detail::once_epoch_mutex);
+			pthread_mutex_lock(&inf.mutex);
 			while (flag <= being_initialized) {
 				if (flag == uninitialized_flag) {
 					flag = being_initialized;
-					pthread_mutex_unlock(&detail::once_epoch_mutex);
+					pthread_mutex_unlock(&inf.mutex);
 					f();
-					pthread_mutex_lock(&detail::once_epoch_mutex);
+					pthread_mutex_lock(&inf.mutex);
 					
-					--detail::once_global_epoch;
-					flag = detail::once_global_epoch;
-					pthread_cond_broadcast(&detail::once_epoch_cv);
+					--inf.flag;
+					flag = inf.flag;
+					pthread_cond_broadcast(&inf.cond);
 				}
 				else {
 					while (flag == being_initialized) {
-						pthread_cond_wait(&detail::once_epoch_cv, &detail::once_epoch_mutex);
+						pthread_cond_wait(&inf.cond, &inf.mutex);
 					}
 				}
 			}
 			
-			this_thread_epoch = detail::once_global_epoch;
-			pthread_mutex_unlock(&detail::once_epoch_mutex);
+			this_thread_epoch = inf.flag;
+			pthread_mutex_unlock(&inf.mutex);
 		}
 	}
 }
